@@ -3,16 +3,21 @@ Internal package providing a Python CRUD interface to MLflow models and versions
 This is a lower level API than the :py:mod:`mlflow.tracking.fluent` module, and is
 exposed in the :py:mod:`mlflow.tracking` module.
 """
+from datetime import timedelta, datetime
+from time import sleep
 
 import logging
 
 from mlflow.exceptions import MlflowException
 from mlflow.store.model_registry import SEARCH_REGISTERED_MODEL_MAX_RESULTS_DEFAULT
 from mlflow.entities.model_registry import RegisteredModelTag, ModelVersionTag
-from mlflow.tracking._model_registry import utils
+from mlflow.entities.model_registry.model_version_status import ModelVersionStatus
+from mlflow.tracking._model_registry import utils, DEFAULT_AWAIT_MAX_SLEEP_SECONDS
 
 _logger = logging.getLogger(__name__)
 
+AWAIT_MODEL_VERSION_CREATE_SLEEP_DURATION_SECONDS = 3
+
 
 class ModelRegistryClient(object):
     """
@@ -29,21 +34,22 @@ class ModelRegistryClient(object):
 
     # Registered Model Methods
 
-    def create_registered_model(self, name, tags=None):
+    def create_registered_model(self, name, tags=None, description=None):
         """
-        Create a new registered model in backend store.
+         Create a new registered model in backend store.
 
-        :param name: Name of the new model. This is expected to be unique in the backend store.
-        :param tags: A dictionary of key-value pairs that are converted into
-                     :py:class:`mlflow.entities.model_registry.RegisteredModelTag` objects.
-        :return: A single object of :py:class:`mlflow.entities.model_registry.RegisteredModel`
-                 created by backend.
+         :param name: Name of the new model. This is expected to be unique in the backend store.
+         :param tags: A dictionary of key-value pairs that are converted into
+                      :py:class:`mlflow.entities.model_registry.RegisteredModelTag` objects.
+        :param description: Description of the model.
+         :return: A single object of :py:class:`mlflow.entities.model_registry.RegisteredModel`
+                  created by backend.
         """
         # TODO: Do we want to validate the name is legit here - non-empty without "/" and ":" ?
         #       Those are constraints applicable to any backend, given the model URI format.
         tags = tags if tags else {}
         tags = [RegisteredModelTag(key, str(value)) for key, value in tags.items()]
-        return self.store.create_registered_model(name, tags)
+        return self.store.create_registered_model(name, tags, description)
 
     def update_registered_model(self, name, description):
         """
@@ -79,9 +85,9 @@ class ModelRegistryClient(object):
         """
         self.store.delete_registered_model(name)
 
-    def list_registered_models(self,
-                               max_results=SEARCH_REGISTERED_MODEL_MAX_RESULTS_DEFAULT,
-                               page_token=None):
+    def list_registered_models(
+        self, max_results=SEARCH_REGISTERED_MODEL_MAX_RESULTS_DEFAULT, page_token=None
+    ):
         """
         List of all registered models.
         :param max_results: Maximum number of registered models desired.
@@ -94,11 +100,13 @@ class ModelRegistryClient(object):
         """
         return self.store.list_registered_models(max_results, page_token)
 
-    def search_registered_models(self,
-                                 filter_string=None,
-                                 max_results=SEARCH_REGISTERED_MODEL_MAX_RESULTS_DEFAULT,
-                                 order_by=None,
-                                 page_token=None):
+    def search_registered_models(
+        self,
+        filter_string=None,
+        max_results=SEARCH_REGISTERED_MODEL_MAX_RESULTS_DEFAULT,
+        order_by=None,
+        page_token=None,
+    ):
         """
         Search for registered models in backend that satisfy the filter criteria.
 
@@ -156,21 +164,57 @@ class ModelRegistryClient(object):
 
     # Model Version Methods
 
-    def create_model_version(self, name, source, run_id, tags=None):
+    def create_model_version(
+        self,
+        name,
+        source,
+        run_id,
+        tags=None,
+        run_link=None,
+        description=None,
+        await_creation_for=DEFAULT_AWAIT_MAX_SLEEP_SECONDS,
+    ):
         """
-        Create a new model version from given source or run ID.
+        Create a new model version from given source.
 
-        :param name: Name ID for containing registered model.
+        :param name: Name of the containing registered model.
         :param source: Source path where the MLflow model is stored.
         :param run_id: Run ID from MLflow tracking server that generated the model.
         :param tags: A dictionary of key-value pairs that are converted into
                      :py:class:`mlflow.entities.model_registry.ModelVersionTag` objects.
+        :param run_link: Link to the run from an MLflow tracking server that generated this model.
+        :param description: Description of the version.
+        :param await_creation_for: Number of seconds to wait for the model version to finish being
+                                    created and is in ``READY`` status. By default, the function
+                                    waits for five minutes. Specify 0 or None to skip waiting.
+        Wait until the model version is finished being created and is in ``READY`` status.
         :return: Single :py:class:`mlflow.entities.model_registry.ModelVersion` object created by
                  backend.
         """
         tags = tags if tags else {}
         tags = [ModelVersionTag(key, str(value)) for key, value in tags.items()]
-        return self.store.create_model_version(name, source, run_id, tags)
+        mv = self.store.create_model_version(name, source, run_id, tags, run_link, description)
+        if await_creation_for and await_creation_for > 0:
+            _logger.info(
+                "Waiting up to %d seconds for model version to finish creation. \
+                    Model name: %s, version %s",
+                await_creation_for,
+                name,
+                mv.version,
+            )
+            max_datetime = datetime.utcnow() + timedelta(seconds=await_creation_for)
+            pending_status = ModelVersionStatus.to_string(ModelVersionStatus.PENDING_REGISTRATION)
+            while mv.status == pending_status:
+                if datetime.utcnow() > max_datetime:
+                    raise MlflowException(
+                        "Exceeded max wait time for model name: {} version: {} to become READY. \
+                            Status: {} Wait Time: {}".format(
+                            mv.name, mv.version, mv.status, await_creation_for
+                        )
+                    )
+                mv = self.get_model_version(mv.name, mv.version)
+                sleep(AWAIT_MODEL_VERSION_CREATE_SLEEP_DURATION_SECONDS)
+        return mv
 
     def update_model_version(self, name, version, description):
         """
@@ -198,8 +242,11 @@ class ModelRegistryClient(object):
         if stage.strip() == "":
             raise MlflowException("The stage must not be an empty string.")
         return self.store.transition_model_version_stage(
-            name=name, version=version, stage=stage,
-            archive_existing_versions=archive_existing_versions)
+            name=name,
+            version=version,
+            stage=stage,
+            archive_existing_versions=archive_existing_versions,
+        )
 
     def get_model_version(self, name, version):
         """
