@@ -12,17 +12,17 @@ from mlflow.utils.proto_json_utils import parse_dict
 from mlflow.utils.string_utils import strip_suffix
 from mlflow.exceptions import MlflowException, RestException
 
-RESOURCE_DOES_NOT_EXIST = 'RESOURCE_DOES_NOT_EXIST'
+_REST_API_PATH_PREFIX = "/api/2.0"
+RESOURCE_DOES_NOT_EXIST = "RESOURCE_DOES_NOT_EXIST"
 
 _logger = logging.getLogger(__name__)
 
-_DEFAULT_HEADERS = {
-    'User-Agent': 'mlflow-python-client/%s' % __version__
-}
+_DEFAULT_HEADERS = {"User-Agent": "mlflow-python-client/%s" % __version__}
 
 
-def http_request(host_creds, endpoint, retries=3, retry_interval=3,
-                 max_rate_limit_interval=60, **kwargs):
+def http_request(
+    host_creds, endpoint, retries=3, retry_interval=3, max_rate_limit_interval=60, **kwargs
+):
     """
     Makes an HTTP request with the specified method to the specified hostname/endpoint. Ratelimit
     error code (429) will be retried with an exponential back off (1, 2, 4, ... seconds) for at most
@@ -44,7 +44,7 @@ def http_request(host_creds, endpoint, retries=3, retry_interval=3,
 
     headers = dict(_DEFAULT_HEADERS)
     if auth_str:
-        headers['Authorization'] = auth_str
+        headers["Authorization"] = auth_str
 
     if host_creds.server_cert_path is None:
         verify = not host_creds.ignore_tls_verification
@@ -52,7 +52,7 @@ def http_request(host_creds, endpoint, retries=3, retry_interval=3,
         verify = host_creds.server_cert_path
 
     if host_creds.client_cert_path is not None:
-        kwargs['cert'] = host_creds.client_cert_path
+        kwargs["cert"] = host_creds.client_cert_path
 
     def request_with_ratelimit_retries(max_rate_limit_interval, **kwargs):
         response = requests.request(**kwargs)
@@ -63,28 +63,36 @@ def http_request(host_creds, endpoint, retries=3, retry_interval=3,
                 "API request to {path} returned status code 429 (Rate limit exceeded). "
                 "Retrying in %d seconds. "
                 "Will continue to retry 429s for up to %d seconds.",
-                sleep, time_left)
+                sleep,
+                time_left,
+            )
             time.sleep(sleep)
             time_left -= sleep
             response = requests.request(**kwargs)
-            sleep = min(time_left, sleep*2)  # sleep for 1, 2, 4, ... seconds;
+            sleep = min(time_left, sleep * 2)  # sleep for 1, 2, 4, ... seconds;
         return response
 
-    cleaned_hostname = strip_suffix(hostname, '/')
+    cleaned_hostname = strip_suffix(hostname, "/")
     url = "%s%s" % (cleaned_hostname, endpoint)
     for i in range(retries):
-        response = request_with_ratelimit_retries(max_rate_limit_interval,
-                                                  url=url, headers=headers, verify=verify, **kwargs)
+        response = request_with_ratelimit_retries(
+            max_rate_limit_interval, url=url, headers=headers, verify=verify, **kwargs
+        )
         if response.status_code >= 200 and response.status_code < 500:
             return response
         else:
             _logger.error(
                 "API request to %s failed with code %s != 200, retrying up to %s more times. "
                 "API response body: %s",
-                url, response.status_code, retries - i - 1, response.text)
+                url,
+                response.status_code,
+                retries - i - 1,
+                response.text,
+            )
             time.sleep(retry_interval)
-    raise MlflowException("API request to %s failed to return code 200 after %s tries" %
-                          (url, retries))
+    raise MlflowException(
+        "API request to %s failed to return code 200 after %s tries" % (url, retries)
+    )
 
 
 def _can_parse_as_json(string):
@@ -104,14 +112,26 @@ def http_request_safe(host_creds, endpoint, **kwargs):
 
 
 def verify_rest_response(response, endpoint):
-    """Verify the return code and raise exception if the request was not successful."""
+    """Verify the return code and format, raise exception if the request was not successful."""
     if response.status_code != 200:
         if _can_parse_as_json(response.text):
             raise RestException(json.loads(response.text))
         else:
-            base_msg = "API request to endpoint %s failed with error code " \
-                       "%s != 200" % (endpoint, response.status_code)
+            base_msg = "API request to endpoint %s failed with error code " "%s != 200" % (
+                endpoint,
+                response.status_code,
+            )
             raise MlflowException("%s. Response body: '%s'" % (base_msg, response.text))
+
+    # Skip validation for endpoints (e.g. DBFS file-download API) which may return a non-JSON
+    # response
+    if endpoint.startswith(_REST_API_PATH_PREFIX) and not _can_parse_as_json(response.text):
+        base_msg = (
+            "API request to endpoint was successful but the response body was not "
+            "in a valid JSON format"
+        )
+        raise MlflowException("%s. Response body: '%s'" % (base_msg, response.text))
+
     return response
 
 
@@ -135,12 +155,14 @@ def call_endpoint(host_creds, endpoint, method, json_body, response_proto):
     # Convert json string to json dictionary, to pass to requests
     if json_body:
         json_body = json.loads(json_body)
-    if method == 'GET':
+    if method == "GET":
         response = http_request(
-            host_creds=host_creds, endpoint=endpoint, method=method, params=json_body)
+            host_creds=host_creds, endpoint=endpoint, method=method, params=json_body
+        )
     else:
         response = http_request(
-            host_creds=host_creds, endpoint=endpoint, method=method, json=json_body)
+            host_creds=host_creds, endpoint=endpoint, method=method, json=json_body
+        )
     response = verify_rest_response(response, endpoint)
     js_dict = json.loads(response.text)
     parse_dict(js_dict=js_dict, message=response_proto)
@@ -169,8 +191,17 @@ class MlflowHostCreds(object):
         function (see https://requests.readthedocs.io/en/master/api/).
         If this is set ``ignore_tls_verification`` must be false.
     """
-    def __init__(self, host, username=None, password=None, token=None,
-                 ignore_tls_verification=False, client_cert_path=None, server_cert_path=None):
+
+    def __init__(
+        self,
+        host,
+        username=None,
+        password=None,
+        token=None,
+        ignore_tls_verification=False,
+        client_cert_path=None,
+        server_cert_path=None,
+    ):
         if not host:
             raise MlflowException(
                 message="host is a required parameter for MlflowHostCreds",
@@ -178,11 +209,13 @@ class MlflowHostCreds(object):
             )
         if ignore_tls_verification and (server_cert_path is not None):
             raise MlflowException(
-                message=("When 'ignore_tls_verification' is true then 'server_cert_path' "
-                         "must not be set! This error may have occurred because the "
-                         "'MLFLOW_TRACKING_INSECURE_TLS' and 'MLFLOW_TRACKING_SERVER_CERT_PATH' "
-                         "environment variables are both set - only one of these environment "
-                         "variables may be set."),
+                message=(
+                    "When 'ignore_tls_verification' is true then 'server_cert_path' "
+                    "must not be set! This error may have occurred because the "
+                    "'MLFLOW_TRACKING_INSECURE_TLS' and 'MLFLOW_TRACKING_SERVER_CERT_PATH' "
+                    "environment variables are both set - only one of these environment "
+                    "variables may be set."
+                ),
                 error_code=INVALID_PARAMETER_VALUE,
             )
         self.host = host
