@@ -1,5 +1,9 @@
 context("Run")
 
+teardown({
+  mlflow_clear_test_dir("mlruns")
+})
+
 test_that("mlflow can run and save model", {
   mlflow_clear_test_dir("mlruns")
 
@@ -61,3 +65,25 @@ test_that("mlflow_run passes all numbers as non-scientific", {
     mlflow_run("project", parameters = c(scientific = 10e4, non_scientific = 30000))
   })
 })
+
+test_that("active experiment is set when starting a run with experiemnt specified", {
+  mlflow_clear_test_dir("mlruns")
+  id <- mlflow_create_experiment("one-more")
+  mlflow_start_run(experiment_id = id)
+  expect_equal(
+    mlflow_get_experiment()$experiment_id,
+    id
+  )
+  mlflow_end_run()
+})
+
+test_that("active experiment is set when starting a run without experiemnt specified", {
+  mlflow_clear_test_dir("mlruns")
+  id <- mlflow_create_experiment("second-exp")
+  mlflow_start_run()
+  expect_equal(
+    mlflow_get_experiment()$experiment_id,
+    "0"
+  )
+  mlflow_end_run()
+})
